@@ -3,7 +3,7 @@
  *
  * minimodem - software audio Bell-type or RTTY FSK modem
  *
- * Copyright (C) 2011-2012 Kamal Mostafa <kamal@whence.com>
+ * Copyright (C) 2011-2013 Kamal Mostafa <kamal@whence.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -72,6 +72,32 @@ tx_stop_transmit_sighandler( int sig )
 /*
  * rudimentary BFSK transmitter
  */
+
+static void fsk_transmit_frame(
+	simpleaudio *sa_out,
+	unsigned int bits,
+	unsigned int n_data_bits,
+	size_t bit_nsamples,
+	float bfsk_mark_f,
+	float bfsk_space_f,
+	float bfsk_nstartbits,
+	float bfsk_nstopbits
+	)
+{
+    int i;
+    if ( bfsk_nstartbits > 0 )
+	simpleaudio_tone(sa_out, bfsk_space_f,
+			bit_nsamples * bfsk_nstartbits);	// start
+    for ( i=0; i<n_data_bits; i++ ) {				// data
+	unsigned int bit = ( bits >> i ) & 1;
+	float tone_freq = bit == 1 ? bfsk_mark_f : bfsk_space_f;
+	simpleaudio_tone(sa_out, tone_freq, bit_nsamples);
+    }
+    if ( bfsk_nstopbits > 0 )
+	simpleaudio_tone(sa_out, bfsk_mark_f,
+			bit_nsamples * bfsk_nstopbits);		// stop
+}
+
 static void fsk_transmit_stdin(
 	simpleaudio *sa_out,
 	int tx_interactive,
@@ -81,6 +107,8 @@ static void fsk_transmit_stdin(
 	int n_data_bits,
 	float bfsk_nstartbits,
 	float bfsk_nstopbits,
+	unsigned int bfsk_do_tx_sync_bytes,
+	unsigned int bfsk_sync_byte,
 	databits_encoder encode
 	)
 {
@@ -115,30 +143,27 @@ static void fsk_transmit_stdin(
 	// fprintf(stderr, "<c=%d>", c);
 	unsigned int nwords;
 	unsigned int bits[2];
+	unsigned int j;
 	nwords = encode(bits, c);
 
 	if ( !tx_transmitting )
 	{
 	    tx_transmitting = 1;
-	    int j;
+	    /* emit leader tone (mark) */
 	    for ( j=0; j<tx_leader_bits_len; j++ )
 		simpleaudio_tone(sa_out, bfsk_mark_f, bit_nsamples);
+	    /* emit "preamble" of sync bytes */
+	    for ( j=0; j<bfsk_do_tx_sync_bytes; j++ )
+		fsk_transmit_frame(sa_out, bfsk_sync_byte, n_data_bits,
+			    bit_nsamples, bfsk_mark_f, bfsk_space_f,
+			    bfsk_nstartbits, bfsk_nstopbits);
 	}
-	unsigned int j;
-	for ( j=0; j<nwords; j++ ) {
-	    if ( bfsk_nstartbits > 0 )
-		simpleaudio_tone(sa_out, bfsk_space_f,
-				bit_nsamples * bfsk_nstartbits);	// start
-	    int i;
-	    for ( i=0; i<n_data_bits; i++ ) {				// data
-		unsigned int bit = ( bits[j] >> i ) & 1;
-		float tone_freq = bit == 1 ? bfsk_mark_f : bfsk_space_f;
-		simpleaudio_tone(sa_out, tone_freq, bit_nsamples);
-	    }
-	    if ( bfsk_nstopbits > 0 )
-		simpleaudio_tone(sa_out, bfsk_mark_f,
-				bit_nsamples * bfsk_nstopbits);		// stop
-	}
+
+	/* emit data bits */
+	for ( j=0; j<nwords; j++ )
+	    fsk_transmit_frame(sa_out, bits[j], n_data_bits,
+			bit_nsamples, bfsk_mark_f, bfsk_space_f,
+			bfsk_nstartbits, bfsk_nstopbits);
 
 	if ( tx_interactive )
 	    setitimer(ITIMER_REAL, &itv, NULL);
@@ -282,7 +307,7 @@ version()
 {
     printf(
     "minimodem %s\n"
-    "Copyright (C) 2011-2012 Kamal Mostafa <kamal@whence.com>\n"
+    "Copyright (C) 2011-2013 Kamal Mostafa <kamal@whence.com>\n"
     "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
     "This is free software: you are free to change and redistribute it.\n"
     "There is NO WARRANTY, to the extent permitted by law.\n\n"
@@ -346,7 +371,8 @@ main( int argc, char*argv[] )
     unsigned int bfsk_inverted_freqs = 0;
     int bfsk_nstartbits = -1;
     float bfsk_nstopbits = -1;
-    unsigned int bfsk_sync_on_data = 0;
+    unsigned int bfsk_do_rx_sync = 0;
+    unsigned int bfsk_do_tx_sync_bytes = 0;
     unsigned int bfsk_sync_byte = -1;
     unsigned int bfsk_n_data_bits = 0;
     int autodetect_shift;
@@ -518,7 +544,8 @@ main( int argc, char*argv[] )
 			assert( bfsk_nstopbits >= 0 );
 			break;
 	    case MINIMODEM_OPT_SYNC_BYTE:
-			bfsk_sync_on_data = 1;
+			bfsk_do_rx_sync = 1;
+			bfsk_do_tx_sync_bytes = 16;
 			bfsk_sync_byte = strtol(optarg, NULL, 0);
 			break;
 	    case 'q':
@@ -621,7 +648,8 @@ main( int argc, char*argv[] )
 	bfsk_n_data_bits = 8;
 	bfsk_nstartbits = 0;
 	bfsk_nstopbits = 0;
-	bfsk_sync_on_data = 1;
+	bfsk_do_rx_sync = 1;
+	bfsk_do_tx_sync_bytes = 16;
 	bfsk_sync_byte = 0xAB;
 	bfsk_mark_f = 2083.0 + 1/3.0;
 	bfsk_space_f = 1562.5;
@@ -741,6 +769,8 @@ main( int argc, char*argv[] )
 				bfsk_n_data_bits,
 				bfsk_nstartbits,
 				bfsk_nstopbits,
+				bfsk_do_tx_sync_bytes,
+				bfsk_sync_byte,
 				bfsk_databits_encode
 				);
 
@@ -974,7 +1004,7 @@ main( int argc, char*argv[] )
 	for ( i=0; i<bfsk_nstartbits; i++ )
 	    expect_bits_string[j++] = '0';
 	for ( i=0; i<bfsk_n_data_bits; i++,j++ ) {
-	    if ( ! carrier && bfsk_sync_on_data )
+	    if ( ! carrier && bfsk_do_rx_sync )
 		expect_bits_string[j] = ( (bfsk_sync_byte>>i)&1 ) + '0';
 	    else
 		expect_bits_string[j] = 'd';
@@ -1194,7 +1224,7 @@ main( int argc, char*argv[] )
 	unsigned int dataout_nbytes = 0;
 
 	// suppress printing of bfsk_sync_byte bytes
-	if ( bfsk_sync_on_data ) {
+	if ( bfsk_do_rx_sync ) {
 	    if ( dataout_nbytes == 0 && bits == bfsk_sync_byte )
 		continue;
 	}
